@@ -1,32 +1,6 @@
 # coding: utf-8
-# /*##########################################################################
-#
-# Copyright (c) 2016-2022 European Synchrotron Radiation Facility
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
-#
-# ###########################################################################*/
-
-__authors__ = ["H. Payno"]
-__license__ = "MIT"
-__date__ = "04/01/2022"
 
+from __future__ import annotations
 
 import logging
 
@@ -42,6 +16,10 @@ from tomoscan.scanbase import ReducedFramesInfos
 
 _logger = logging.getLogger(__name__)
 
+__all__ = [
+    "HDF5FrameReducer",
+]
+
 
 class HDF5FrameReducer(FrameReducerBase):
     """Frame reducer dedicated to HDF5"""
@@ -54,7 +32,7 @@ class HDF5FrameReducer(FrameReducerBase):
         :param NXtomoScan scan: scan containing frames to reduce
         :param REDUCER_TARGET target: dark of flat to be reduced
         """
-        target = REDUCER_TARGET.from_value(target)
+        target = REDUCER_TARGET(target)
         if target is REDUCER_TARGET.DARKS:
             raw_what = scan.darks
         elif target is REDUCER_TARGET.FLATS:
@@ -87,11 +65,11 @@ class HDF5FrameReducer(FrameReducerBase):
         else:
             return self.scan.count_time[indexes]
 
-    def get_machine_electric_current(self, indexes):
-        if self.scan.electric_current is None:
+    def get_machine_current(self, indexes):
+        if self.scan.machine_current is None:
             return []
         else:
-            return self.scan.electric_current[indexes]
+            return self.scan.machine_current[indexes]
 
     def load_data_serie(self, urls) -> dict:
         """load all urls. Trying to reduce load time by calling get_compacted_dataslices"""
@@ -177,43 +155,43 @@ class HDF5FrameReducer(FrameReducerBase):
         # res: key is serie index (first serie frame index), value is the numpy.array of the reduced frame
         infos = ReducedFramesInfos()
 
-        for serie_ in raw_series:
-            serie_index = min(serie_)
+        for series_ in raw_series:
+            series_index = min(series_)
             if self.reducer_target is REDUCER_TARGET.DARKS and len(res) > 0:
                 continue
 
-            serie_frame_data = self.load_data_serie(serie_)
-            serie_count_time = self.get_count_time_serie(indexes=list(serie_.keys()))
-            serie_machine_electric_current = self.get_machine_electric_current(
-                indexes=list(serie_.keys())
+            series_frame_data = self.load_data_serie(series_)
+            series_count_time = self.get_count_time_serie(indexes=list(series_.keys()))
+            series_machine_current = self.get_machine_current(
+                indexes=list(series_.keys())
             )
 
             if method_ == "raw":
                 # i method is raw then only the targetted frame (first or last) will be loaded
-                data = res[serie_index] = serie_frame_data.reshape(
-                    -1, serie_frame_data.shape[-1]
+                data = res[series_index] = series_frame_data.reshape(
+                    -1, series_frame_data.shape[-1]
                 )
                 if self.reduced_method is ReduceMethod.FIRST:
                     index_infos = 0
                 elif self.reduced_method is ReduceMethod.LAST:
                     index_infos = -1
-                if len(serie_machine_electric_current) > 0:
-                    infos.machine_electric_current.append(
-                        serie_machine_electric_current[index_infos]
+                else:
+                    raise ValueError(
+                        f"reduced method not handle for method 'raw' ({self.reduced_method}). Expects 'first' or 'last'"
                     )
-                if len(serie_count_time) > 0:
-                    infos.count_time.append(serie_count_time[index_infos])
+                if len(series_machine_current) > 0:
+                    infos.machine_current.append(series_machine_current[index_infos])
+                if len(series_count_time) > 0:
+                    infos.count_time.append(series_count_time[index_infos])
             else:
-                data = method_(serie_frame_data, axis=0)
-                if len(serie_machine_electric_current) > 0:
-                    infos.machine_electric_current.append(
-                        method_(serie_machine_electric_current)
-                    )
-                if len(serie_count_time) > 0:
-                    infos.count_time.append(method_(serie_count_time))
+                data = method_(series_frame_data, axis=0)
+                if len(series_machine_current) > 0:
+                    infos.machine_current.append(method_(series_machine_current))
+                if len(series_count_time) > 0:
+                    infos.count_time.append(method_(series_count_time))
 
             if self.output_dtype is not None:
                 data = data.astype(self.output_dtype)
-            res[serie_index] = data
+            res[series_index] = data
 
         return res, infos
