@@ -1320,14 +1320,24 @@ class NXtomoScan(TomoScanBase):
                     "incoherent number of rotation angle vs sequence number"
                 )
 
-            for i_frame, rot_a, img_key, x_tr, y_tr, z_tr, i_m in zip(
-                sequence_number,
-                rotation_angles,
-                image_keys,
-                x_translation,
-                y_translation,
-                z_translation,
-                intensity_monitor,
+            for i_frame, (
+                frame_seq_num,
+                rot_a,
+                img_key,
+                x_tr,
+                y_tr,
+                z_tr,
+                i_m,
+            ) in enumerate(
+                zip(
+                    sequence_number,
+                    rotation_angles,
+                    image_keys,
+                    x_translation,
+                    y_translation,
+                    z_translation,
+                    intensity_monitor,
+                )
             ):
                 url = DataUrl(
                     file_path=self.master_file,
@@ -1338,6 +1348,7 @@ class NXtomoScan(TomoScanBase):
 
                 frame = TomoFrame(
                     index=i_frame,
+                    sequence_number=frame_seq_num,
                     url=url,
                     image_key=img_key,
                     rotation_angle=rot_a,
@@ -1352,9 +1363,9 @@ class NXtomoScan(TomoScanBase):
                             ImageKey(int(self.image_key_control[frame.index]))
                             is ImageKey.ALIGNMENT
                         )
-                    except Exception:
+                    except KeyError:
                         _logger.warning(
-                            f"Unable to deduce if {frame.index} is a control frame. Consider it is not"
+                            f"Unable to deduce if frame number {frame.index} is a control frame. Consider it is not"
                         )
                         is_control_frame = False
                 else:
@@ -1821,6 +1832,7 @@ class TomoFrame:
     def __init__(
         self,
         index: int,
+        sequence_number: int,
         url: DataUrl | None = None,
         image_key: None | ImageKey | int = None,
         rotation_angle: float | None = None,
@@ -1835,6 +1847,7 @@ class TomoFrame:
                 f"index is expected to be an instance of {int}. Got {type(index)}"
             )
         self._index = index
+        self._sequence_number = sequence_number
         if image_key is not None:
             self._image_key = ImageKey(image_key)
         else:
@@ -1850,8 +1863,14 @@ class TomoFrame:
 
     @property
     def index(self) -> int:
+        """Index of the frame in the .nx file"""
         return self._index
 
+    @property
+    def sequence_number(self) -> int:
+        """Sequence number of the frame in the acquisition !!! can be different from the index"""
+        return self._sequence_number
+
     @property
     def image_key(self) -> ImageKey:
         return self._image_key
