@@ -566,7 +566,7 @@ def test_NXtomoScan_source_API():
 
 def test_TomoFrame_API():
     """Test TomoFrame API"""
-    frame = TomoFrame(index=0)
+    frame = TomoFrame(index=0, sequence_number=0)
     frame.image_key = ImageKey.PROJECTION
     with pytest.raises(TypeError):
         frame.image_key = "projection"
@@ -652,12 +652,19 @@ def test_save_and_load_dark(tmp_path):
         numpy.testing.assert_array_equal(loaded_darks[0], dark_frame)
 
         # 2. test overwrite and make sure we are properly removing the existing dataset
-        output_file_path = scan._format_file_path(
-            url=scan.REDUCED_DARKS_DATAURLS[0],
-            entry=scan.entry,
-            idx=0,
-            idx_zfill4=str(0).zfill(4),
+        output_file_name = (
+            scan.REDUCED_DARKS_DATAURLS[0]
+            .file_path()
+            .format(
+                entry=scan.entry,
+                idx=0,
+                idx_zfill4=str(0).zfill(4),
+                scan_prefix=os.path.basename(
+                    scan.path,
+                ),
+            )
         )
+        output_file_path = os.path.join(scan.path, output_file_name)
 
         # 2.1 make sure when should overwrite the dataset the adding size is limited (and dataset is overwrite)
         first_file_size = os.path.getsize(output_file_path)
@@ -1107,7 +1114,7 @@ def test_splitted_flat_series(tmp_path):
 
     setImageKey([ImageKey.PROJECTION.value] * 10)
     nx_tomo_reader._image_keys = nx_tomo_reader._image_keys_control = [
-        ImageKey.PROJECTION
+        ImageKey.PROJECTION.value
     ] * 10
     assert len(nx_tomo_reader.splitted_flat_series) == 0
 
