@@ -1,38 +1,10 @@
 # coding: utf-8
-# /*##########################################################################
-#
-# Copyright (c) 2016-2022 European Synchrotron Radiation Facility
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
-#
-# ###########################################################################*/
 """module defining utils for a jp2k volume"""
 
-
-__authors__ = ["H. Payno", "P. Paleo"]
-__license__ = "MIT"
-__date__ = "27/01/2022"
-
+from __future__ import annotations
 
 import logging
 import os
-from typing import Optional
 
 import numpy
 from packaging.version import parse as parse_version
@@ -67,19 +39,28 @@ _logger = logging.getLogger(__name__)
 
 _MISSING_GLYMUR_MSG = "Fail to import glymur. won't be able to load / save volume to jp2k. You can install it by calling pip."
 
+__all__ = [
+    "JP2KVolume",
+    "has_glymur",
+    "has_minimal_openjpeg",
+    "glymur_version",
+    "openjpeg_version",
+    "get_available_threads",
+]
+
 
 class JP2KVolume(VolumeSingleFrameBase):
     """
     Save volume data to single frame jp2k files and metadata to .txt file
 
-    :param Optional[list] cratios: list of ints. compression ratio for each jpeg2000 layer
-    :param Optional[list] psnr: list of int.
+    :param cratios: list of ints. compression ratio for each jpeg2000 layer
+    :param psnr: list of int.
                                 The PSNR (Peak Signal-to-Noise ratio) for each jpeg2000 layer.
                                 This defines a quality metric for lossy compression.
                                 The number "0" stands for lossless compression.
-    :param Optional[int] n_threads: number of thread to use for writing. If None will try to get as much as possible
-    :param Optional[tuple] clip_values: optional tuple of two float (min, max) to clamp volume value
-    :param bool rescale_data: rescale data before dumping each frame. Expected to be True when dump a new volume and False
+    :param n_threads: number of thread to use for writing. If None will try to get as much as possible
+    :param clip_values: optional tuple of two float (min, max) to clamp volume value
+    :param rescale_data: rescale data before dumping each frame. Expected to be True when dump a new volume and False
                               when save volume cast for example (and when histogram is know...)
 
     :warning: each file saved under {volume_basename}_{index_zfill6}.jp2k is considered to be a slice of the volume.
@@ -91,21 +72,21 @@ class JP2KVolume(VolumeSingleFrameBase):
 
     def __init__(
         self,
-        folder: Optional[str] = None,
-        volume_basename: Optional[str] = None,
-        data: Optional[numpy.ndarray] = None,
-        source_scan: Optional[TomoScanBase] = None,
-        metadata: Optional[dict] = None,
-        data_url: Optional[DataUrl] = None,
-        metadata_url: Optional[DataUrl] = None,
+        folder: str | None = None,
+        volume_basename: str | None = None,
+        data: numpy.ndarray | None = None,
+        source_scan: TomoScanBase | None = None,
+        metadata: dict | None = None,
+        data_url: DataUrl | None = None,
+        metadata_url: DataUrl | None = None,
         overwrite: bool = False,
         start_index=0,
         data_extension=DEFAULT_DATA_EXTENSION,
         metadata_extension=VolumeSingleFrameBase.DEFAULT_METADATA_EXTENSION,
-        cratios: Optional[list] = None,
-        psnr: Optional[list] = None,
-        n_threads: Optional[int] = None,
-        clip_values: Optional[tuple] = None,
+        cratios: list | None = None,
+        psnr: list | None = None,
+        n_threads: int | None = None,
+        clip_values: tuple | None = None,
         rescale_data: bool = True,
     ) -> None:
         if folder is not None:
@@ -147,19 +128,19 @@ class JP2KVolume(VolumeSingleFrameBase):
         self.clip_values = clip_values  # execute test about the type...
 
     @property
-    def cratios(self) -> Optional[list]:
+    def cratios(self) -> list | None:
         return self._cratios
 
     @cratios.setter
-    def cratios(self, cratios: Optional[list]):
+    def cratios(self, cratios: list | None):
         self._cratios = cratios
 
     @property
-    def psnr(self) -> Optional[list]:
+    def psnr(self) -> list | None:
         return self._psnr
 
     @psnr.setter
-    def psnr(self, psnr: Optional[list]):
+    def psnr(self, psnr: list | None):
         self._psnr = psnr
 
     @property
@@ -173,15 +154,14 @@ class JP2KVolume(VolumeSingleFrameBase):
         self._rescale_data = rescale
 
     @property
-    def clip_values(self) -> Optional[tuple]:
+    def clip_values(self) -> tuple | None:
         """
         :return: optional min and max value to clip - as float.
-        :rtype: Optional[tuple]
         """
         return self._clip_values
 
     @clip_values.setter
-    def clip_values(self, values: Optional[tuple]) -> None:
+    def clip_values(self, values: tuple | None) -> None:
         if values is None:
             self._clip_values = None
         elif not isinstance(values, (tuple, list)):
@@ -194,7 +174,7 @@ class JP2KVolume(VolumeSingleFrameBase):
             self._clip_values = values
 
     @docstring(VolumeSingleFrameBase)
-    def save_data(self, url: Optional[DataUrl] = None) -> None:
+    def save_data(self, url: DataUrl | None = None) -> None:
         self._cast_already_logged = False
         super().save_data(url=url)
 
@@ -203,13 +183,16 @@ class JP2KVolume(VolumeSingleFrameBase):
         if not has_glymur:
             raise RuntimeError(_MISSING_GLYMUR_MSG)
 
+        expected_output_dtype = frame.dtype
         if self.clip_values is not None:
             frame = numpy.clip(frame, self.clip_values[0], self.clip_values[1])
+        if expected_output_dtype not in (numpy.uint8, numpy.uint16):
+            _logger.info(
+                f"{self.get_identifier().to_str()} get {frame.dtype}. Cast it as {numpy.uint16}"
+            )
+            expected_output_dtype = numpy.uint16
         if self.rescale_data:
-            if frame.dtype in (numpy.uint8, numpy.uint16):
-                max_uint = numpy.iinfo(frame.dtype).max
-            else:
-                max_uint = numpy.iinfo(numpy.uint16).max
+            max_uint = numpy.iinfo(expected_output_dtype).max
             frame = rescale_data(
                 data=frame,
                 new_min=0,
@@ -217,13 +200,7 @@ class JP2KVolume(VolumeSingleFrameBase):
                 data_min=self.clip_values[0] if self.clip_values is not None else None,
                 data_max=self.clip_values[1] if self.clip_values is not None else None,
             )
-        if not isinstance(frame.dtype, (numpy.uint8, numpy.uint16)):
-            if self._cast_already_logged:
-                self._cast_already_logged = True
-                _logger.info(
-                    f"{self.get_identifier().to_str()} get {frame.dtype}. Cast it as {numpy.uint16}"
-                )
-            frame = frame.astype(numpy.uint16)
+        frame = frame.astype(expected_output_dtype)
 
         if scheme == "glymur":
             glymur.Jp2k(file_name, data=frame, psnr=self.psnr, cratios=self.cratios)
@@ -242,19 +219,18 @@ class JP2KVolume(VolumeSingleFrameBase):
             raise ValueError(f"Scheme {scheme} is not handled")
 
     @staticmethod
-    def setup_multithread_encoding(n_threads=None, what_if_not_available="ignore"):
+    def setup_multithread_encoding(
+        n_threads: int | None = None, what_if_not_available: str = "ignore"
+    ):
         """
         Setup OpenJpeg multi-threaded encoding.
 
-        Parameters
-        -----------
-        n_threads: int, optional
-            Number of threads. If not provided, all available threads are used.
-        na: str, optional
-            What to do if requirements are not fulfilled. Possible values are:
-               - "ignore": do nothing, proceed
-               - "print": show an information message
-               - "raise": raise an error
+        :param n_threads: Number of threads. If not provided, all available threads are used.
+        :param what_if_not_available: What to do if requirements are not fulfilled. Possible values are:
+            - "ignore": do nothing, proceed
+            - "print": show an information message
+            - "raise": raise an error
+
         """
         required_glymur_version = "0.9.3"
         required_openjpeg_version = "2.4.0"
