@@ -18,6 +18,7 @@ from tomoscan.esrf.identifier.rawidentifier import RawVolumeIdentifier
 from tomoscan.scanbase import TomoScanBase
 from tomoscan.utils import docstring
 from tomoscan.volumebase import VolumeBase
+from tomoscan.io import cast_to_default_types
 
 _logger = logging.getLogger(__name__)
 
@@ -241,7 +242,8 @@ class RawVolume(VolumeBase):
             _logger.info(f"save data to {metadata_file}")
             if len(self.metadata) > 0:
                 # same as ini but no section. Write works but read fails
-                dicttoini(self.metadata, metadata_file)
+                metadata = cast_to_default_types(self.metadata)
+                dicttoini(metadata, metadata_file)
         elif url.scheme() == "lxml":
             metadata_file = url.file_path()
             _logger.info(f"save data to {metadata_file}")
