@@ -16,6 +16,7 @@ from silx.io.url import DataUrl
 from tomoscan.scanbase import TomoScanBase
 from tomoscan.utils import docstring
 from tomoscan.volumebase import VolumeBase, SliceTuple
+from tomoscan.io import cast_to_default_types
 
 _logger = logging.getLogger(__name__)
 
@@ -199,7 +200,8 @@ class VolumeSingleFrameBase(VolumeBase):
                     metadata_file = os.path.join(metadata_file, url.data_path())
                     _logger.info(f"save data to {metadata_file}")
                     if len(self.metadata) > 0:
-                        dicttoini(self.metadata, metadata_file)
+                        metadata = cast_to_default_types(self.metadata)
+                        dicttoini(metadata, metadata_file)
             else:
                 raise ValueError(f"scheme {url.scheme()} is not handled")
 
