@@ -21,6 +21,7 @@ from tomoscan.esrf.volume.singleframebase import (
 from tomoscan.scanbase import TomoScanBase
 from tomoscan.utils import docstring, get_subvolume_shape
 from tomoscan.volumebase import VolumeBase, SliceTuple
+from tomoscan.io import cast_to_default_types
 
 try:
     import tifffile  # noqa #F401 needed for later possible lazy loading
@@ -343,8 +344,9 @@ class MultiTIFFVolume(VolumeBase):
             if url.data_path() is not None:
                 raise ValueError("data_path is not handled by 'ini' scheme")
             else:
+                metadata = cast_to_default_types(self.metadata)
                 dicttoini(
-                    self.metadata,
+                    metadata,
                     url.file_path(),
                 )
         else:
