@@ -21,6 +21,7 @@ from silx.io.url import DataUrl
 from silx.io.utils import get_data
 from silx.io.utils import open as hdf5_open
 from tomoscan.utils.io import deprecated, deprecated_warning
+from tomoscan.framereducer.ReduceFrameSaver import ReduceFrameSaver
 
 from tomoscan.identifier import ScanIdentifier
 from tomoscan.normalization import IntensityNormalization
@@ -1153,24 +1154,9 @@ class TomoScanBase(TomoObject):
     def get_dataset_basename(self) -> str:
         raise NotImplementedError("Base class")
 
-    def _format_file_path(self, url, entry, idx, idx_zfill4):
-        file_path = url.file_path()
-        if file_path is not None:
-            file_path = file_path.format(
-                index=str(idx),
-                index_zfill4=idx_zfill4,
-                entry=entry,
-                scan_prefix=self.get_dataset_basename(),
-            )
-
-        if not os.path.isabs(file_path):
-            file_path = os.path.join(self.path, file_path)
-
-        return file_path
-
     def _dump_frame_dict(
         self,
-        frames: dict,
+        frames: dict[int, numpy.ndarray],
         output_urls,
         frames_metadata: ReducedFramesInfos | None,
         metadata_output_urls: tuple | None,
@@ -1182,183 +1168,60 @@ class TomoScanBase(TomoObject):
         Behavior with HDF5: it expects to have a dedicated group where it can save the different frame with indices.
         It will do a first iteration at this group level to remove unused dataset and will overwrite the one he can in order to reduced memory print
         """
-        if not isinstance(frames, dict):
-            raise TypeError(
-                f"inputs `frames` is expected to be a dict not {type(frames)}"
-            )
-        if not isinstance(output_urls, (list, tuple, set)):
-            raise TypeError(
-                f"output_urls is expected to be a tuple not a {type(output_urls)}"
-            )
-        if self.path is None:
-            raise ValueError("No dataset path provided")
+        saver = ReduceFrameSaver(
+            frames=frames,
+            output_urls=[self._format_url(url) for url in output_urls],
+            frames_metadata=frames_metadata,
+            metadata_output_urls=[
+                self._format_url(url) for url in metadata_output_urls
+            ],
+            overwrite=overwrite,
+        )
+        saver.save()
 
-        if frames_metadata is not None:
-            if not isinstance(frames_metadata, ReducedFramesInfos):
-                raise TypeError(
-                    f"darks_infos is a {type(frames_metadata)} when None or {ReducedFramesInfos} expected"
-                )
-            self._check_reduced_infos(reduced_frames=frames, infos=frames_metadata)
+    def _format_url(self, url: DataUrl):
+        """
+        Format an url according to the scan metadata. Solves ''entry' and 'scan_prefix' keywords
+        """
+        assert isinstance(
+            url, DataUrl
+        ), f"url should be an instance of {DataUrl}. Got {type(url)}"
+        if hasattr(self, "entry"):
+            entry = self.entry
+        else:
+            entry = "entry"
 
-        def format_data_path(url, entry, idx, idx_zfill4):
-            data_path = url.data_path()
+        def format_data_path(data_path) -> str:
+            """Format only 'entry' field. This is up to the 'ReduceFrameSaver' to take care of 'index' and 'index_zfill4'"""
             if data_path is not None:
                 data_path = data_path.format(
-                    index=str(idx), index_zfill4=idx_zfill4, entry=entry
+                    entry=entry, index="{index}", index_zfill4="{index_zfill4}"
                 )
             return data_path
 
-        entry = "entry"
-        if hasattr(self, "entry"):
-            entry = self.entry
+        def format_file_path(file_path) -> str:
+            """Format only 'entry' and 'scan_prefix' field. This is up to the 'ReduceFrameSaver' to take care of 'index' and 'index_zfill4'"""
+            if file_path is not None:
+                if not os.path.isabs(file_path):
+                    if self.path is None:
+                        raise ValueError("No dataset path provided")
 
-        def clean_frame_group(url):
-            """
-            For HDF5 in order to avoid file size increase we need to overwrite dataset when possible.
-            But the darks / flats groups can contain other datasets and pollute this group.
-            This function will remove unused dataset (frame index) when necessary
-            """
-            file_path = self._format_file_path(
-                url, entry=entry, idx=None, idx_zfill4=None
-            )
-            if not (os.path.exists(file_path) and h5py.is_hdf5(file_path)):
-                return
+                    file_path = os.path.join(self.path, file_path)
 
-            group_path = "/".join(
-                format_data_path(url, entry=entry, idx=0, idx_zfill4="0000").split("/")[
-                    :-1
-                ]
-            )
-            used_datasets = []
-            for idx, _ in frames.items():
-                idx_zfill4 = str(idx).zfill(4)
-                used_datasets.append(
-                    format_data_path(
-                        url, entry=entry, idx=idx, idx_zfill4=idx_zfill4
-                    ).split("/")[-1]
+                file_path = file_path.format(
+                    entry=entry,
+                    scan_prefix=self.get_dataset_basename(),
+                    index="{index}",
+                    index_zfill4="{index_zfill4}",
                 )
-            with h5py.File(file_path, mode="a") as h5s:
-                if group_path in h5s:
-                    if not overwrite:
-                        raise KeyError("group_path already exists")
-                    for key in h5s[group_path].keys():
-                        if key not in used_datasets:
-                            del h5s[group_path][key]
+            return file_path
 
-        # save data
-        for url in output_urls:
-            clean_frame_group(url=url)
-            # first delete keys that are no more used
-            for i_frame, (idx, frame) in enumerate(frames.items()):
-                if not isinstance(frame, numpy.ndarray):
-                    raise TypeError("frames are expected to be 2D numpy.ndarray")
-                elif frame.ndim == 3 and frame.shape[0] == 1:
-                    frame = frame.reshape([frame.shape[1], frame.shape[2]])
-                elif frame.ndim != 2:
-                    raise ValueError("frames are expected to be 2D numpy.ndarray")
-                idx_zfill4 = str(idx).zfill(4)
-                data_path = format_data_path(
-                    url, entry=entry, idx=idx, idx_zfill4=idx_zfill4
-                )
-
-                file_path = self._format_file_path(
-                    url, entry=entry, idx=idx, idx_zfill4=idx_zfill4
-                )
-
-            # small hack to insure 'flats' or 'darks' group are cleaned when start to write in
-            for i_frame, (idx, frame) in enumerate(frames.items()):
-                if not isinstance(frame, numpy.ndarray):
-                    raise TypeError("frames are expected to be 2D numpy.ndarray")
-                elif frame.ndim == 3 and frame.shape[0] == 1:
-                    frame = frame.reshape([frame.shape[1], frame.shape[2]])
-                elif frame.ndim != 2:
-                    raise ValueError("frames are expected to be 2D numpy.ndarray")
-                idx_zfill4 = str(idx).zfill(4)
-                data_path = format_data_path(
-                    url, entry=entry, idx=idx, idx_zfill4=idx_zfill4
-                )
-
-                file_path = self._format_file_path(
-                    url, entry=entry, idx=idx, idx_zfill4=idx_zfill4
-                )
-                scheme = url.scheme()
-
-                if scheme == "fabio":
-                    if data_path is not None:
-                        raise ValueError("fabio does not handle data_path")
-                    else:
-                        # for edf: add metadata to the header if some, without taking into account the
-                        # metadata_output_urls (too complicated for backward compatibility...)
-                        header = {}
-                        if (
-                            frames_metadata is not None
-                            and len(frames_metadata.machine_current) > 0
-                        ):
-                            header["SRCUR"] = frames_metadata.machine_current[i_frame]
-                        if (
-                            frames_metadata is not None
-                            and len(frames_metadata.count_time) > 0
-                        ):
-                            header["CountTime"] = frames_metadata.count_time[i_frame]
-
-                        edf_writer = fabio.edfimage.EdfImage(
-                            data=frame,
-                            header=header,
-                        )
-                        edf_writer.write(file_path)
-                elif scheme in ("hdf5", "silx"):
-                    os.makedirs(os.path.dirname(file_path), exist_ok=True)
-                    with h5py.File(file_path, mode="a") as h5s:
-                        if data_path in h5s:
-                            h5s[data_path][()] = frame
-                        else:
-                            h5s[data_path] = frame
-                        h5s[data_path].attrs["interpretation"] = "image"
-                else:
-                    raise ValueError(
-                        f"scheme {scheme} is not handled for frames. Should be fabio, silx of hdf5"
-                    )
-
-        frames_indexes = [idx for idx, _ in frames.items()]
-        if frames_metadata is not None:
-            for url, idx in zip(metadata_output_urls, frames_indexes):
-                idx_zfill4 = str(idx).zfill(4)
-                metadata_grp_path = format_data_path(
-                    url, entry=entry, idx=idx, idx_zfill4=idx_zfill4
-                )
-                file_path = self._format_file_path(
-                    url, entry=entry, idx=idx, idx_zfill4=idx_zfill4
-                )
-                scheme = url.scheme()
-                for metadata_name, metadata_values in frames_metadata.to_dict().items():
-                    # warning: for now we only handle list (of count_time and machine_current)
-                    if len(metadata_values) == 0:
-                        continue
-                    else:
-                        # save metadata
-                        if scheme in ("hdf5", "silx"):
-                            with h5py.File(file_path, mode="a") as h5s:
-                                metadata_path = "/".join(
-                                    [metadata_grp_path, metadata_name]
-                                )
-                                if metadata_path in h5s:
-                                    del h5s[metadata_path]
-                                h5s[metadata_path] = metadata_values
-                                unit = None
-                                if metadata_name == ReducedFramesInfos.COUNT_TIME_KEY:
-                                    unit = _ureg.second
-                                elif (
-                                    metadata_name
-                                    == ReducedFramesInfos.MACHINE_ELECT_CURRENT_KEY
-                                ):
-                                    unit = _ureg.ampere
-                                if unit is not None:
-                                    h5s[metadata_path].attrs["units"] = str(unit)
-
-                        else:
-                            raise ValueError(
-                                f"scheme {scheme} is not handled for frames metadata. Should be silx of hdf5"
-                            )
+        return DataUrl(
+            file_path=format_file_path(url.file_path()),
+            data_path=format_data_path(url.data_path()),
+            scheme=url.scheme(),
+            data_slice=url.data_slice(),
+        )
 
     def _load_frame_dict(
         self, inputs_urls, metadata_input_urls, return_as_url=False
@@ -1598,19 +1461,6 @@ class TomoScanBase(TomoObject):
 
         return res_data, res_metadata
 
-    @staticmethod
-    def _check_reduced_infos(reduced_frames, infos):
-        incoherent_metadata_mess = "incoherent provided infos:"
-        incoherent_metadata = False
-        if len(infos.count_time) not in (0, len(reduced_frames)):
-            incoherent_metadata = True
-            incoherent_metadata_mess += f"\n - count_time gets {len(infos.count_time)} when 0 or {len(reduced_frames)} expected"
-        if len(infos.machine_current) not in (0, len(reduced_frames)):
-            incoherent_metadata = True
-            incoherent_metadata_mess += f"\n - machine_current gets {len(infos.machine_current)} when 0 or {len(reduced_frames)} expected"
-        if incoherent_metadata:
-            raise ValueError(incoherent_metadata_mess)
-
     def save_reduced_darks(
         self,
         darks: dict,
