@@ -1,66 +1,50 @@
 # coding: utf-8
-# /*##########################################################################
-#
-# Copyright (c) 2016-2022 European Synchrotron Radiation Facility
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
-#
-# ###########################################################################*/
+
 """module to define base class for a volume"""
 
+from __future__ import annotations
 
-__authors__ = ["H. Payno"]
-__license__ = "MIT"
-__date__ = "27/01/2022"
-
-
-from typing import Optional, Union
+import pint
+from typing import NamedTuple
 import logging
 import numpy
 from silx.io.url import DataUrl
 from silx.math.combo import (  # pylint: disable=E0611 (not found from the pyx file)
     min_max,
 )
-from pyunitsystem.metricsystem import MetricSystem
 
 from tomoscan.utils.io import deprecated, deprecated_warning
 from tomoscan.identifier import VolumeIdentifier
 from tomoscan.scanbase import TomoScanBase
 from tomoscan.tomoobject import TomoObject
 from tomoscan.utils import BoundingBox1D, BoundingBox3D
+from tomoscan.utils import drac as drac_utils
 
+_ureg = pint.get_application_registry()
 _logger = logging.getLogger(__name__)
 
 
+__all__ = ["SliceTuple", "VolumeBase"]
+
+
+class SliceTuple(NamedTuple):
+    axis: int
+    index: int
+
+
 class VolumeBase(TomoObject):
     """
     context: we aim at having a common way of saving and loading volumes through the tomotools suite.
     The goal is to aim handling of volumes when creating them or doing some operations with those like stitching...
 
-    :param DataUlr url: url of the volume. Could be path to a master file if we can provide one per each volume. Otherwise could be a pattern of edf files or tiff file with a data range
-    :param Optional[TomoScanBase] source_scan: potential instance of TomoScanBase in order to get extra information. This could be saved in the volume file to (external link)
-    :param Optional[nump.ndarray] data: volume data. Expected to be 3D
-    :param Optional[dict] metadata: metadata associated to the volume. Must be a dict of serializable object
-    :param Optional[DataUrl] data_url: url to save the data. If provided url must not be provided. If an object is constructed from data and metadta url then no rule to create a VolumeIdentifier can be created and call to et_identifier will raise an error.
-    :param Optional[DataUrl] metadata_url: url to save the metadata. If provided url must not be provided. If an object is constructed from data and metadta url then no rule to create a VolumeIdentifier can be created and call to et_identifier will raise an error.
-    :param bool overwrite: when save the data if encounter a ressource already existing overwrite it (if True) or not.
-    :param str overwrite: when save the data if encounter a ressource already existing overwrite it (if True) or not.
+    :param url: url of the volume. Could be path to a master file if we can provide one per each volume. Otherwise could be a pattern of edf files or tiff file with a data range
+    :param source_scan: potential instance of TomoScanBase in order to get extra information. This could be saved in the volume file to (external link)
+    :param data: volume data. Expected to be 3D
+    :param metadata: metadata associated to the volume. Must be a dict of serializable object
+    :param data_url: url to save the data. If provided url must not be provided. If an object is constructed from data and metadata url then no rule to create a VolumeIdentifier can be created and call to et_identifier will raise an error.
+    :param metadata_url: url to save the metadata. If provided url must not be provided. If an object is constructed from data and metadata url then no rule to create a VolumeIdentifier can be created and call to et_identifier will raise an error.
+    :param overwrite: when save the data if encounter a resource already existing overwrite it (if True) or not.
+    :param overwrite: when save the data if encounter a resource already existing overwrite it (if True) or not.
 
     :raises TypeError:
     :raises ValueError: * if data is a numpy array and not 3D.
@@ -71,15 +55,15 @@ class VolumeBase(TomoObject):
 
     def __init__(
         self,
-        url: Optional[DataUrl] = None,
-        data: Optional[numpy.ndarray] = None,
-        source_scan: Optional[TomoScanBase] = None,
-        metadata: Optional[dict] = None,
-        data_url: Optional[DataUrl] = None,
-        metadata_url: Optional[DataUrl] = None,
+        url: DataUrl | None = None,
+        data: numpy.ndarray | None = None,
+        source_scan: TomoScanBase | None = None,
+        metadata: dict | None = None,
+        data_url: DataUrl | None = None,
+        metadata_url: DataUrl | None = None,
         overwrite: bool = False,
-        data_extension: Optional[str] = None,
-        metadata_extension: Optional[str] = None,
+        data_extension: str | None = None,
+        metadata_extension: str | None = None,
     ) -> None:
         super().__init__()
         if url is not None and (data_url is not None or metadata_url is not None):
@@ -107,7 +91,7 @@ class VolumeBase(TomoObject):
             self._data_url = data_url
             self._metadata_url = metadata_url
         else:
-            # otherwise have be setted when url has been set from call to deduce_data_and_metadata_urls
+            # otherwise have be set from call to 'deduce_data_and_metadata_urls'
             pass
 
     @property
@@ -115,16 +99,16 @@ class VolumeBase(TomoObject):
         return self._url
 
     @url.setter
-    def url(self, url: Optional[DataUrl]) -> None:
+    def url(self, url: DataUrl | None) -> None:
         if url is not None and not isinstance(url, DataUrl):
             raise TypeError
         self._url = url
         self._data_url, self._metadata_url = self.deduce_data_and_metadata_urls(url)
 
-    def deduce_data_and_metadata_urls(self, url: Optional[DataUrl]) -> tuple:
+    def deduce_data_and_metadata_urls(self, url: DataUrl | None) -> tuple:
         """
         compute data and metadata urls from 'parent url'
-        :return: data_url: Optional[DataUrl], metadata_url: Optional[DataUrl]
+        :return: data_url: DataUrl | None, metadata_url: DataUrl | None
         """
         raise NotImplementedError("Base class")
 
@@ -145,7 +129,7 @@ class VolumeBase(TomoObject):
         return self._metadata_url
 
     @property
-    def data(self) -> Optional[numpy.ndarray]:
+    def data(self) -> numpy.ndarray | None:
         return self._data
 
     @data.setter
@@ -160,13 +144,14 @@ class VolumeBase(TomoObject):
 
     def get_slice(
         self,
-        index: Union[str, int] = None,
+        index: str | int = None,
         axis=None,
         xy=None,
         xz=None,
         yz=None,
-        url: Optional[DataUrl] = None,
+        url: DataUrl | None = None,
     ):
+        "read a single slice of the volume"
         if xy is yz is xz is axis is None:
             raise ValueError("axis should be provided")
 
@@ -192,12 +177,58 @@ class VolumeBase(TomoObject):
         else:
             return None
 
+    def get_slices(
+        self, slices: tuple[SliceTuple | tuple]
+    ) -> dict[SliceTuple, numpy.ndarray]:
+        """
+        retrieve a couple of slices along any axis:
+
+        For example, if you want to retrieve slice number 2 of axis 0 and slice number 56 of axis 1:
+
+        .. code-block:: python
+
+            slices = volume.get_slices(
+                (0, 2),
+                (1, 56),
+            )
+            for (axis, slice), data in slices:
+                ...
+        """
+        if not isinstance(slices, tuple):
+            raise TypeError(f"slices should be a tuple. Got {type(slices)}")
+        for elmt in slices:
+            if not isinstance(elmt, (tuple, NamedTuple)):
+                raise TypeError(
+                    f"slices is expected to be a tuple of tuple or a tuple of SliceTuple. Get {type(elmt)} as an element of the tuple"
+                )
+
+        # cast 2 element tuple to SliceTuple
+        slices = tuple([SliceTuple(*slice_) for slice_ in slices])
+        if self.data is not None:
+            return self.select_slices(volume=self.data, slices=slices)
+        else:
+            return self._get_slices_from_disk(slices=slices)
+
+    def _get_slices_from_disk(
+        self, slices: tuple[SliceTuple], url: DataUrl | None = None
+    ) -> dict[SliceTuple, numpy.ndarray]:
+        """
+        read from files a couple of slices along any axis.
+        """
+
+        data = self.load_data(url=url, store=False)
+        if data is None:
+            raise RuntimeError(
+                f"Enable to load data from disk for {self.get_identifier()}"
+            )
+        return self.select_slices(volume=data, slices=slices)
+
     @property
-    def metadata(self) -> Optional[dict]:
+    def metadata(self) -> dict | None:
         return self._metadata
 
     @metadata.setter
-    def metadata(self, metadata: Optional[dict]):
+    def metadata(self, metadata: dict | None):
         if not isinstance(metadata, (dict, type(None))):
             raise TypeError(
                 f"metadata is expected to be None or a dict not {type(metadata)}"
@@ -210,13 +241,13 @@ class VolumeBase(TomoObject):
         raise NotImplementedError("Base class")
 
     def clear_cache(self):
-        """remove object stored in data and medatada"""
+        """remove object stored in data and metadata"""
         self.data = None
         self.metadata = None
 
     # generic function requested
     @property
-    def source_scan(self) -> Optional[TomoScanBase]:
+    def source_scan(self) -> TomoScanBase | None:
         return self.__source_scan
 
     @property
@@ -231,7 +262,7 @@ class VolumeBase(TomoObject):
 
     # function to be loaded to an url
     @staticmethod
-    def from_identifier(identifier: Union[str, VolumeIdentifier]):
+    def from_identifier(identifier: str | VolumeIdentifier):
         """Return the Dataset from a identifier"""
         raise NotImplementedError("Base class")
 
@@ -250,7 +281,7 @@ class VolumeBase(TomoObject):
             ddict["processing_options"]["reconstruction"] = {}
 
     @property
-    def position(self) -> Optional[tuple]:
+    def position(self) -> tuple | None:
         """position are provided as a tuple using the same reference for axis as the volume data.
         position is returned as (axis_0_pos, axis_1_pos, axis_2_pos). Can also be see as (z_position, y_position, x_position)
         """
@@ -290,9 +321,10 @@ class VolumeBase(TomoObject):
         self.voxel_size = pixel_size
 
     @property
-    def voxel_size(self) -> Optional[tuple]:
+    def voxel_size(self) -> tuple[float] | None:
         """
         voxel size as (axis 0 dim - aka z, axis 1 dim - aka y, axis 2 dim aka z)
+        Returned in meter
         """
         metadata = self.metadata or self.load_metadata()
 
@@ -309,14 +341,15 @@ class VolumeBase(TomoObject):
                 .get("reconstruction", {})
                 .get("pixel_size_cm", None)
             )
-            if voxel_size is not None:
-                voxel_size = (voxel_size, voxel_size, voxel_size)
 
         if voxel_size is not None:
             if numpy.isscalar(voxel_size):
                 # be safer and handle the case voxel size is a scalar
                 voxel_size = [voxel_size] * 3
-            return tuple(numpy.array(voxel_size) * MetricSystem.CENTIMETER.value)
+            # voxel are in cm -> convert them to meter
+            voxel_size = (voxel_size * _ureg.centimeter).to(_ureg.meter).magnitude
+            # convert to float
+            return tuple([float(value) for value in voxel_size])
         return None
 
     @voxel_size.setter
@@ -326,14 +359,17 @@ class VolumeBase(TomoObject):
         if numpy.isscalar(voxel_size):
             raise TypeError("voxel is expected to be a tuple of three values")
         self._insure_reconstruction_dict_exists(self.metadata)
+        voxel_size_m = voxel_size * _ureg.meter
+        voxel_size_cm = voxel_size_m.to(_ureg.centimeter)
         self.metadata["processing_options"]["reconstruction"]["voxel_size_cm"] = (
-            numpy.array(voxel_size) / MetricSystem.CENTIMETER.value
+            numpy.array(voxel_size_cm.magnitude)
         )
 
-    def get_volume_shape(self, url=None):
+    def get_volume_shape(self, url=None) -> tuple:
+        """return volume shape as a tuple"""
         raise NotImplementedError("Base class")
 
-    def get_bounding_box(self, axis: Optional[Union[str, int]] = None):
+    def get_bounding_box(self, axis: str | int | None = None):
         if axis is None:
             x_bb = self.get_bounding_box(axis=2)
             y_bb = self.get_bounding_box(axis=1)
@@ -399,7 +435,8 @@ class VolumeBase(TomoObject):
         # it in order to load data next
         self.load_data(store=True)
 
-    def save(self, url: Optional[DataUrl] = None, **kwargs):
+    def save(self, url: DataUrl | None = None, **kwargs):
+        """save volume data and metadata to disk"""
         if url is not None:
             data_url, metadata_url = self.deduce_data_and_metadata_urls(url=url)
         else:
@@ -411,24 +448,26 @@ class VolumeBase(TomoObject):
             # if no metadata found
             self.save_metadata(metadata_url)
 
-    def save_data(self, url: Optional[DataUrl] = None, **kwargs) -> None:
+    def save_data(self, url: DataUrl | None = None, **kwargs) -> None:
         """
         save data to the provided url or existing one if none is provided
         """
         raise NotImplementedError("Base class")
 
-    def save_metadata(self, url: Optional[DataUrl] = None) -> None:
+    def save_metadata(self, url: DataUrl | None = None) -> None:
         """
         save metadata to the provided url or existing one if none is provided
         """
         raise NotImplementedError("Base class")
 
     def load_data(
-        self, url: Optional[DataUrl] = None, store: bool = True
+        self, url: DataUrl | None = None, store: bool = True
     ) -> numpy.ndarray:
+        """load volume data from disk"""
         raise NotImplementedError("Base class")
 
-    def load_metadata(self, url: Optional[DataUrl] = None, store: bool = True) -> dict:
+    def load_metadata(self, url: DataUrl | None = None, store: bool = True) -> dict:
+        """load volume metadata from disk"""
         raise NotImplementedError("Base class")
 
     def check_can_provide_identifier(self):
@@ -438,7 +477,26 @@ class VolumeBase(TomoObject):
             )
 
     @staticmethod
-    def select(volume, xy=None, xz=None, yz=None, axis=None, index=None):
+    def select_slices(volume: numpy.ndarray, slices: tuple[SliceTuple]) -> dict:
+        return {
+            SliceTuple(axis=slice_.axis, index=slice_.index): VolumeBase.select(
+                volume=volume, index=slice_.index, axis=slice_.axis
+            )
+            for slice_ in slices
+        }
+
+    @staticmethod
+    def select(
+        volume: numpy.ndarray,
+        xy: int | None = None,
+        xz: int | None = None,
+        yz: int | None = None,
+        axis: int | None = None,
+        index: int | None = None,
+    ) -> numpy.array:
+        """
+        select a slice at 'index' along an axis (axis)
+        """
         if xy is not None:
             deprecated_warning(
                 type_="parameter",
@@ -488,7 +546,7 @@ class VolumeBase(TomoObject):
         """
         :param url: data url. If not provided will take self.data_url
 
-        return a generator go through all the existings files associated to the data volume
+        return a generator go through all the existing files associated to the data volume
         """
         raise NotImplementedError("Base class")
 
@@ -496,7 +554,7 @@ class VolumeBase(TomoObject):
         """
         :param url: metadata url. If not provided will take self.metadata_url
 
-        return a generator go through all the existings files associated to the data volume
+        return a generator go through all the existing files associated to the data volume
         """
         raise NotImplementedError("Base class")
 
@@ -554,8 +612,17 @@ class VolumeBase(TomoObject):
         Provide a helper class to dump data frame by frame. For know the only possible interaction is
         Helper[:] = frame
 
-        :param int n_frames: number of frame the final volume will contain
+        :param n_frames: number of frame the final volume will contain
         :param DataUrl data_url: url to dump data
-        :param bool overwrite: overwrite existing file ?
+        :param overwrite: overwrite existing file ?
         """
         raise NotImplementedError("Base class")
+
+    def build_drac_metadata(self) -> dict:
+        """
+        build the drac (successor of icat) metadata dict from existing volume metadata.
+        """
+        metadata = self.metadata or self.load_metadata()
+        flatten_metadata = drac_utils.flatten_vol_metadata(metadata)
+        icat_metadata = drac_utils.map_nabu_keys_to_drac(flatten_metadata)
+        return icat_metadata
